

#include <stdint.h>


using namespace std;


#define  RCU_APB2EN  *((volatile uint32_t *) 0x40021018)
#define  GPIOC_CTL1  *((volatile uint32_t *) 0x40011004)
#define  GPIOC_OCTL  *((volatile uint32_t *) 0x4001100C)
#define  MTIME       *((volatile uint64_t *) 0xD1000000)
#define  MTIMECMP    *((volatile uint64_t *) 0xD1000008)

#define  MTIME_1_SEC  24000000     // 24 MHz, so for 1 sec we need 24000000 cycles


#define  CLIC_BASE                    0xD2000000
#define  CLIC_IP(source)              *(volatile uint8_t *)(CLIC_BASE + 0x00001000 + ((source) * 4))
#define  CLIC_IE(source)              *(volatile uint8_t *)(CLIC_BASE + 0x00001001 + ((source) * 4))
#define  CLIC_ATTR(source)            *(volatile uint8_t *)(CLIC_BASE + 0x00001002 + ((source) * 4))
#define  CLIC_CTL(source)             *(volatile uint8_t *)(CLIC_BASE + 0x00001003 + ((source) * 4))
#define  CLIC_CFG                     *(volatile uint8_t *)(CLIC_BASE + 0x00000000)
#define  CLIC_MTH                     *(volatile uint8_t *)(CLIC_BASE + 0x0000000B)
#define  CLIC_IP_IE_ATTR_CTL(source)  *(volatile uint32_t *)(CLIC_BASE + 0x00001000 + ((source) * 4))


void clicIntTmr() __attribute__ ((interrupt, section(".clic_int_tmr")));


void clicIntTmr() {
	MTIME = 0;
	MTIMECMP = MTIME_1_SEC;
	GPIOC_OCTL = GPIOC_OCTL ^ (((uint32_t) 1) << 13);   // toggle led
	CLIC_IP(7) = 0;    // not pending
}


int main() {
	// enable clock on port C
	RCU_APB2EN = RCU_APB2EN | (((uint32_t) 1) << 4);
	// PC13 pin is output, low speed, push-pull
	GPIOC_CTL1 = 0x44244444;
	// clear CLIC config register and set vectored interrupts
	CLIC_CFG = 0;
	CLIC_MTH = 0;   // specific register for GD32VF103 ECLIC
	for (uint16_t i = 0; i < 4096; i++)
		CLIC_IP_IE_ATTR_CTL(i) = 0;
	// use CLIC vectored interrupt handler (put 1 on mode (lower two) bits of mtvec)
	asm volatile (
		"csrw mtvec, %[reg]" : : [reg] "r" (0x00000003)
	);
	// machine interrupt enable
	asm volatile (
		"csrsi mstatus, 8"
	);
	// set CLIC interrupt vector table (mtvt = 0x307)
	asm volatile (
		"csrw 0x307, %[reg]" : : [reg] "r" (0x08000000)
	);
	CLIC_ATTR(7) = 0b11000001;    // machine mode, level triggered, vectored
	CLIC_IE(7) = 1;               // enable interrupt 7
	CLIC_IP(7) = 0;               // not pending
	CLIC_CTL(7) = 0;              // priority
	// configure interrupt period
	MTIME = 0;
	MTIMECMP = MTIME_1_SEC;
	while (true)
		asm volatile ("wfi");
}
